﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;
using Microsoft.Practices.Prism.Commands;
using TileManagerWithBinding.Annotations;

namespace TileManagerWithBinding
{
    public interface IScriptInvestigations
    {
        bool IsInScriptMode { get; set; }
        bool Tile3IsMaximized { get; set; }
        bool Tile2IsMaximized { get; set; }
        bool Tile1IsMaximized { get; set; }
        bool Tile4IsMaximized { get; set; }
        int MaximizedTileLimit { get; set; }
        string NavigateForwardText { get; set; }
        string NavigateBackwardText { get; set; }
        bool CanNavigateBackward { get; set; }
    }

    public class MyViewModel : IScriptInvestigations, INotifyPropertyChanged
    {
        private bool _isInScriptMode;
        private bool _tile3IsMaximized;
        private bool _tile2IsMaximized;
        private bool _tile1IsMaximized;
        private bool _tile4IsMaximized;
        private string _navigateForwardText = "Begin Investigation Script";
        private string _navigateBackwardText = "Back";
        private bool _canNavigateBackward;
        private int _maximizedTileLimit = 3;

        private readonly ScriptManager _scriptManager;

        public MyViewModel()
        {

            _scriptManager = new ScriptManager(this);

            NavigateForward = new DelegateCommand(OnNavigateForward);

            NavigateBackwards = new DelegateCommand(OnNavigateBackward);
        }

        private void OnNavigateBackward()
        {
            _scriptManager.NavigateBackward();
        }

        private void OnNavigateForward()
        {
            _scriptManager.NavigateForward();
        }

        public ICommand NavigateForward { get; private set; }

        public ICommand NavigateBackwards { get; private set; }

        public bool IsInScriptMode
        {
            get { return _isInScriptMode; }
            set
            {
                if (value.Equals(_isInScriptMode)) return;
                _isInScriptMode = value;
                OnPropertyChanged("IsInScriptMode");
            }
        }

        public bool Tile3IsMaximized
        {
            get { return _tile3IsMaximized; }
            set
            {
                if (value.Equals(_tile3IsMaximized)) return;
                _tile3IsMaximized = value;
                OnPropertyChanged("Tile3IsMaximized");
            }
        }

        public bool Tile2IsMaximized
        {
            get { return _tile2IsMaximized; }
            set
            {
                if (value.Equals(_tile2IsMaximized)) return;
                _tile2IsMaximized = value;
                OnPropertyChanged("Tile2IsMaximized");
            }
        }

        public bool Tile1IsMaximized
        {
            get { return _tile1IsMaximized; }
            set
            {
                if (value.Equals(_tile1IsMaximized)) return;
                _tile1IsMaximized = value;
                OnPropertyChanged("Tile1IsMaximized");
            }
        }

        public bool Tile4IsMaximized
        {
            get { return _tile4IsMaximized; }
            set
            {
                if (value.Equals(_tile4IsMaximized)) return;
                _tile4IsMaximized = value;
                OnPropertyChanged("Tile4IsMaximized");
            }
        }

        public string NavigateForwardText
        {
            get { return _navigateForwardText; }
            set
            {
                if (value == _navigateForwardText) return;
                _navigateForwardText = value;
                OnPropertyChanged("NavigateForwardText");
            }
        }

        public string NavigateBackwardText
        {
            get { return _navigateBackwardText; }
            set
            {
                if (value == _navigateBackwardText) return;
                _navigateBackwardText = value;
                OnPropertyChanged("NavigateBackwardText");
            }
        }

        public bool CanNavigateBackward
        {
            get { return _canNavigateBackward; }
            set
            {
                if (value.Equals(_canNavigateBackward)) return;
                _canNavigateBackward = value;
                OnPropertyChanged("CanNavigateBackward");
            }
        }

        public int MaximizedTileLimit
        {
            get { return _maximizedTileLimit; }
            set
            {
                if (value.Equals(_maximizedTileLimit)) return;
                _maximizedTileLimit = value;
                OnPropertyChanged("MaximizedTileLimit");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }

    internal class ScriptManager
    {
        private readonly IScriptInvestigations _viewModel;
        private int _scriptStep;

        public ScriptManager(IScriptInvestigations viewModel)
        {
            _viewModel = viewModel;
        }

        internal void NavigateForward()
        {
            switch (_scriptStep)
            {
                case 0:
                    //_viewModel.MaximizedTileLimit = 1;
                    _scriptStep = 1;
                    _viewModel.NavigateForwardText = "Next";
                    //_viewModel.Tile4IsMaximized = false;
                    //_viewModel.Tile2IsMaximized = false;
                    //_viewModel.Tile3IsMaximized = false;
                    _viewModel.Tile1IsMaximized = true;
                    break;
                case 1:
                    _scriptStep = 2;
                    _viewModel.NavigateForwardText = "Next";
                    //_viewModel.Tile1IsMaximized = false;
                    //_viewModel.Tile4IsMaximized = false;
                    //_viewModel.Tile2IsMaximized = false;
                    //_viewModel.Tile3IsMaximized = false;
                    _viewModel.Tile2IsMaximized = true;
                    break;
                case 2:
                    _scriptStep = 3;
                    _viewModel.NavigateForwardText = "See Investigation Result";
                    //_viewModel.Tile1IsMaximized = false;
                    //_viewModel.Tile4IsMaximized = false;
                    //_viewModel.Tile2IsMaximized = false;
                    _viewModel.Tile3IsMaximized = true;
                    break;
                case 3:
                    _scriptStep = 4;
                    _viewModel.NavigateForwardText = "Finish";
                    //_viewModel.Tile1IsMaximized = false;
                    //_viewModel.Tile2IsMaximized = false;
                    //_viewModel.Tile3IsMaximized = false;
                    _viewModel.Tile4IsMaximized = true;
                    break;
                case 4:
                    _scriptStep = 0;
                    //_viewModel.Tile1IsMaximized = false;
                    _viewModel.Tile4IsMaximized = false;
                    //_viewModel.Tile2IsMaximized = false;
                    //_viewModel.Tile3IsMaximized = false;
                    _viewModel.NavigateForwardText = "Begin Investigation Script";
                    break;
            }
        }

        internal void NavigateBackward()
        {
            switch (_scriptStep)
            {
                case 1:
                    _viewModel.IsInScriptMode = false;
                    _scriptStep = 0;
                    _viewModel.NavigateForwardText = "Begin Investigation Script";
                    break;
                case 2:
                    _scriptStep = 1;
                     _viewModel.Tile4IsMaximized = false;
                    _viewModel.Tile2IsMaximized = false;
                    _viewModel.Tile1IsMaximized = false;
                    _viewModel.Tile3IsMaximized = true;
                    break;
                case 3:
                    _scriptStep = 2;
                    _viewModel.Tile4IsMaximized = false;
                    _viewModel.Tile3IsMaximized = false;
                    _viewModel.Tile2IsMaximized = false;
                    _viewModel.Tile1IsMaximized = true;
                    break;
                case 4:
                    _scriptStep = 3;
                    _viewModel.Tile1IsMaximized = false;
                    _viewModel.Tile4IsMaximized = false;
                    _viewModel.Tile3IsMaximized = false;
                    _viewModel.Tile2IsMaximized = true;
                    break;
            }
        }


    }
}
